@@ -21,6 +21,7 @@ from collections import defaultdict
 import http.client
 import urllib.request
 import urllib.parse
+import socket
 
 # Set up logging
 logging.basicConfig(
@@ -109,6 +110,24 @@ class GitChange:
         # Default to the first directory name
         return parts[0]
 
+    @property
+    def is_formatting_change(self) -> bool:
+        """Determine if this change is likely just formatting."""
+        if not self.content_diff:
+            return False
+        
+        # Simple heuristics to detect formatting changes
+        formatting_indicators = [
+            # Only whitespace changes
+            self.content_diff.strip().startswith('diff') and all(line.startswith(('+', '-', ' ')) and line.strip() in ('', '+', '-') for line in self.content_diff.splitlines()[1:] if line and not line.startswith(('---', '+++', 'diff', 'index', '@@'))),
+            # Common formatter markers
+            'import format' in self.content_diff.lower(),
+            'prettier' in self.content_diff.lower(),
+            'fmt' in self.content_diff.lower() and len(self.content_diff) < 500
+        ]
+        
+        return any(formatting_indicators)
+
 
 @dataclass
 class CommitGroup:
@@ -207,19 +226,30 @@ class OllamaClient:
     
     def _get_host_connection(self) -> Tuple[str, int]:
         """Parse host string and return connection parameters."""
-        if self.host.startswith("http://"):
-            parsed_url = urllib.parse.urlparse(self.host)
-            host = parsed_url.netloc
-            port = parsed_url.port or 11434
-        elif self.host.startswith("https://"):
-            parsed_url = urllib.parse.urlparse(self.host)
-            host = parsed_url.netloc
-            port = parsed_url.port or 443
-        else:
-            host = self.host
-            port = 11434
+        try:
+            if self.host.startswith("http://"):
+                parsed_url = urllib.parse.urlparse(self.host)
+                host = parsed_url.netloc.split(':')[0]  # Extract only hostname part
+                port = parsed_url.port or 11434
+            elif self.host.startswith("https://"):
+                parsed_url = urllib.parse.urlparse(self.host)
+                host = parsed_url.netloc.split(':')[0]  # Extract only hostname part
+                port = parsed_url.port or 443
+            else:
+                host = self.host.split(':')[0]  # Handle case if port is included
+                port = 11434
             
-        return host, port
+            # Test connection before returning
+            socket.getaddrinfo(host, port)
+            return host, port
+        except Exception as e:
+            logger.warning(f"Connection error to {self.host}: {str(e)}")
+            # Fall back to localhost if specified host fails
+            if self.host != "localhost" and self.host != "http://localhost:11434":
+                logger.info("Trying localhost as fallback")
+                self.host = "http://localhost:11434"
+                return "localhost", 11434
+            raise
     
     def _get_available_models(self) -> List[str]:
         """Get a list of available models from Ollama."""
@@ -351,7 +381,9 @@ class SmartGitCommitWorkflow:
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
             cwd=self.repo_path,
-            text=True
+            text=True,
+            encoding='utf-8',  # Specify UTF-8 encoding
+            errors='ignore'    # Ignore decoding errors
         )
         stdout, stderr = process.communicate()
         if process.returncode != 0 and stderr:
@@ -488,7 +520,7 @@ class SmartGitCommitWorkflow:
         else:
             # For untracked files, read a sample of the content
             try:
-                with open(os.path.join(self.repo_path, change.filename), 'r') as f:
+                with open(os.path.join(self.repo_path, change.filename), 'r', encoding='utf-8', errors='ignore') as f:
                     file_content = "".join(f.readlines()[:50])
             except Exception:
                 pass
@@ -600,7 +632,7 @@ class SmartGitCommitWorkflow:
         4. The criteria for which files should be included
         
         Format each group as JSON:
-        {"type": "...", "name": "...", "description": "...", "criteria": "..."}
+        {{"type": "...", "name": "...", "description": "...", "criteria": "..."}}
         
         Separate each group with ---
         """
@@ -851,10 +883,21 @@ class SmartGitCommitWorkflow:
                     return
                     
             # Execute the commit
-            with open(os.path.join(self.repo_path, ".git", "COMMIT_EDITMSG"), "w") as f:
-                f.write(commit_message)
-                
-            stdout, code = self._run_git_command(["commit", "-F", os.path.join(".git", "COMMIT_EDITMSG")])
+            # Write commit message with UTF-8 encoding explicitly
+            commit_msg_path = os.path.join(self.repo_path, ".git", "COMMIT_EDITMSG")
+            try:
+                with open(commit_msg_path, "w", encoding='utf-8') as f:
+                    f.write(commit_message)
+                    
+                stdout, code = self._run_git_command(["commit", "-F", os.path.join(".git", "COMMIT_EDITMSG")])
+            finally:
+                # Clean up the temporary commit message file
+                if os.path.exists(commit_msg_path):
+                    try:
+                        os.remove(commit_msg_path)
+                    except OSError as e:
+                        logger.warning(f"Could not remove temporary commit message file: {e}")
+
             if code != 0:
                 logger.error("Failed to commit changes")
                 if interactive:
